// Title:	BiDiPro DLL module
// Desc:	1801PP1
// Ver:		0.0.1
// Found:	2010-09-02
// Last:	????-??-??
// CPU:		386+
// Model:	Tiny
// Compil:	Borland C++ 3.1 (DOS)
// Author:	Oleg O. Chukaev aka 0leg



/*
 | EEPROM	BiDiPro		Port
 | D0, AD1..AD7	A0..A7		1A
 | AD8..AD15	D0..D7		1B
 | -CE		CE		2A4
 | -OE		OE		2A5
 | -CS		A16		2A0
 | WR/RD	A18		2A2
 | Upr		A19		2A3
 | RPLY		A8		1C0
 | Vcc		VCC1		2B7
 | GND		GND		---
 */


#include <stdlib.h>
#include "../dlldata.h"
#include "../errors.h"
#include "../bidi.h"

#ifdef LINUXXX
 extern DLLDATA *dlp;
#else
# define dlp ((DLLDATA *) NULL)		/* ⥫    */
#endif

#define bdport dlp->bdprt		/* 㪠⥫  hardware ࠩ */


/* ७   CFG: */
static struct CFGKEY cfgkeys[] = {
	{'D', "tppw", 50, 50, 100000L},			//Write time (us)
	{'D', "tepw1", 50, 50, 100000L},		//Erase time 1 (us)
	{'D', "tepw2", 100000L, 100000L, 1000000L},	//Erase time 2 (us)
	{'D', "erase", 1, 1, 1L},			//Erase enabled?
	{0, {0}, 0, 0, 0},				//End of table
};


#define cfg_tppw	cfgkeys[0].data
#define cfg_tepw1	cfgkeys[1].data
#define cfg_tepw2	cfgkeys[2].data
#define cfg_erase	cfgkeys[3].data

/* Port 2A */
#define	PIN_CS		0x01
#define	PIN_WRRD	0x04
#define	PIN_UPR		0x08
#define	PIN_CE		0x10
#define	PIN_OE		0x20


static char *copyright="1801PP1 EEPROM module (c) Oleg O. Chukaev  ["__DATE__"]";


/*
 | 몫筨 ⠭
 */
void power_off (void) {
	bdport (PRST);
}


/*
 | 筨 ⠭
 */
int power_on (char mode) {

	if (mode == 'R') {
		bdport (PFORCE | PWR1 | PCIN);		// DD1:OOI
		bdport (PFORCE | PWR2);			// DD4:OOO
		dlp->voltage (dlp->Vcc, 0, 0);
		bdport (PWB2 | 0x80);			// Vcc pin 36
		bdport (PWA2 | BITTER | PIN_OE | PIN_CE | PIN_UPR
			| PIN_WRRD | PIN_CS);
	} else if (mode == 'W' || mode == 'E') {
		bdport (PFORCE | PWR1 | PBIN);		// DD1:OIO
		bdport (PFORCE | PWR2);			// DD4:OOO
		dlp->voltage (65, dlp->Vpp, dlp->Vid);
		bdport (PWB2 | 0x80);			// Vcc pin 36
		bdport (PWA2 | BITTER | PIN_OE);
	}
	dlp->mcsdelay (100000L);
	return 0;
}


/*
 | ⥭, ࠢ, ஢ઠ 
 */
int read_all (void) {

	dword adr;
	dword istep = dlp->esize / 64L;
	dword errors = 0;
	byte rd, rdl, rdh;

	power_on ('R');
	bdport (PWA2 | BITTER | PIN_OE | PIN_CE | PIN_UPR | PIN_WRRD);
	for (adr = 0; adr < dlp->esize; adr += 2) {
		bdport (PWA2 | BITTER | PIN_OE | PIN_CE | PIN_UPR | PIN_WRRD);
		bdport (PWA1 | ((adr >> 0L) & 0xff));	//A0 .. A7
		bdport (PWB1 | ((adr >> 8L) & 0xff));	//A8 .. A15
		bdport (PWA2 | BITTER | PIN_OE          | PIN_UPR | PIN_WRRD);
		bdport (PFORCE | PWR1 | PAIN | PBIN | PCIN);	//III
		bdport (PWA2 | BITTER                   | PIN_UPR | PIN_WRRD);
		dlp->mcsdelay (10L);
		rdl = bdport (PRA1);
		rdh = bdport (PRB1);
		rd = bdport (PRC1);
		bdport (PWA2 | BITTER | PIN_OE          | PIN_UPR | PIN_WRRD);
		bdport (PFORCE | PWR1 | PCIN);			//OOI
		bdport (PWA2 | BITTER | PIN_OE | PIN_CE | PIN_UPR | PIN_WRRD);

		switch (dlp->funcode) {
		case DFN_FREAD:
//			dlp->setbytebuf (adr, rd & 1);
			dlp->setbytebuf (adr, rdl);
			dlp->setbytebuf (adr+1, rdh);
			break;
		case DFN_FCOMPARE:
			if (dlp->getbytebuf (adr) != rd) {
				dlp->setbmark (adr, 1);
				errors++;
			}
			break;
		case DFN_FBLANK:
			if (rd != 0xff) {
				dlp->adr = adr;
				power_off ();
				return ERC_BLANK;
			}
		}

		if ((adr % istep == 0) && (dlp->iprogress (adr, 2))) {
			break;
		}
	}
	dlp->edata = errors;
	power_off ();
	return 0;
}


/*
 | ࠭
 */
int erase (void) {

	return (0);
}


/*
 | 
 */
int write_all (void) {

	return 0;
}



/*
 | Main code
 */
int dmain (void) {

	switch (dlp->funcode) {
	case DFN_LOAD:
		/* ⨢஢ 㯭 㭪樨: */
		dlp->funmask[DFN_INIT] = 1;
		dlp->funmask[DFN_FREAD] = 1;
		dlp->funmask[DFN_FWRITE] = 1;
		dlp->funmask[DFN_ERASE] = 0;
		dlp->funmask[DFN_FCOMPARE] = 0;
		dlp->funmask[DFN_READID] = 0;
		dlp->funmask[DFN_FBLANK] = 0;
		dlp->funmask[DFN_TEST] = 0;
		dlp->defstate = 0xff;
		dlp->copyright = (byte *)copyright;
		dlp->version = DLLVERSION;
		dlp->cfgkeys = cfgkeys;
		return 0;

	case DFN_INIT:
		dlp->dippins = 24;
		return 0;

	case DFN_FREAD:
		return read_all ();

	case DFN_FWRITE:
		return write_all ();

	case DFN_FCOMPARE:
		return read_all ();

	case DFN_FBLANK:
		return read_all ();

	case DFN_ERASE:
		return erase ();

	default:
		return ERC_FUNUNSP;
	}
}

/* E0F */
